@@ -1,9 +1,4 @@
-'''utility functions for printing version information
-
-
-see pandas/pandas/util/_print_versions.py'''
-
-import codecs
+"""Utility functions for printing version information."""
 import importlib
 import locale
 import os
@@ -14,7 +9,7 @@ import sys
 
 
 def get_sys_info():
-    "Returns system information as a dict"
+    """Returns system information as a dict"""
 
     blob = []
 
@@ -22,40 +17,42 @@ def get_sys_info():
     commit = None
     if os.path.isdir(".git") and os.path.isdir("xarray"):
         try:
-            pipe = subprocess.Popen('git log --format="%H" -n 1'.split(" "),
-                                    stdout=subprocess.PIPE,
-                                    stderr=subprocess.PIPE)
-            so, serr = pipe.communicate()
+            pipe = subprocess.Popen(
+                'git log --format="%H" -n 1'.split(" "),
+                stdout=subprocess.PIPE,
+                stderr=subprocess.PIPE,
+            )
+            so, _ = pipe.communicate()
         except Exception:
             pass
         else:
             if pipe.returncode == 0:
                 commit = so
                 try:
-                    commit = so.decode('utf-8')
+                    commit = so.decode("utf-8")
                 except ValueError:
                     pass
                 commit = commit.strip().strip('"')
 
-    blob.append(('commit', commit))
+    blob.append(("commit", commit))
 
     try:
-        (sysname, nodename, release,
-         version, machine, processor) = platform.uname()
-        blob.extend([
-            ("python", sys.version),
-            ("python-bits", struct.calcsize("P") * 8),
-            ("OS", "%s" % (sysname)),
-            ("OS-release", "%s" % (release)),
-            # ("Version", "%s" % (version)),
-            ("machine", "%s" % (machine)),
-            ("processor", "%s" % (processor)),
-            ("byteorder", "%s" % sys.byteorder),
-            ("LC_ALL", "%s" % os.environ.get('LC_ALL', "None")),
-            ("LANG", "%s" % os.environ.get('LANG', "None")),
-            ("LOCALE", "%s.%s" % locale.getlocale()),
-
-        ])
+        (sysname, _nodename, release, _version, machine, processor) = platform.uname()
+        blob.extend(
+            [
+                ("python", sys.version),
+                ("python-bits", struct.calcsize("P") * 8),
+                ("OS", "%s" % (sysname)),
+                ("OS-release", "%s" % (release)),
+                # ("Version", "%s" % (version)),
+                ("machine", "%s" % (machine)),
+                ("processor", "%s" % (processor)),
+                ("byteorder", "%s" % sys.byteorder),
+                ("LC_ALL", "%s" % os.environ.get("LC_ALL", "None")),
+                ("LANG", "%s" % os.environ.get("LANG", "None")),
+                ("LOCALE", "%s.%s" % locale.getlocale()),
+            ]
+        )
     except Exception:
         pass
 
@@ -67,21 +64,33 @@ def netcdf_and_hdf5_versions():
     libnetcdf_version = None
     try:
         import netCDF4
+
         libhdf5_version = netCDF4.__hdf5libversion__
         libnetcdf_version = netCDF4.__netcdf4libversion__
     except ImportError:
         try:
             import h5py
-            libhdf5_version = h5py.__hdf5libversion__
+
+            libhdf5_version = h5py.version.hdf5_version
         except ImportError:
             pass
-    return [('libhdf5', libhdf5_version), ('libnetcdf', libnetcdf_version)]
+    return [("libhdf5", libhdf5_version), ("libnetcdf", libnetcdf_version)]
 
 
-def show_versions(as_json=False):
+def show_versions(file=sys.stdout):
+    """ print the versions of xarray and its dependencies
+
+    Parameters
+    ----------
+    file : file-like, optional
+        print to the given file-like object. Defaults to sys.stdout.
+    """
     sys_info = get_sys_info()
 
-    sys_info.extend(netcdf_and_hdf5_versions())
+    try:
+        sys_info.extend(netcdf_and_hdf5_versions())
+    except Exception as e:
+        print(f"Error collecting netcdf / hdf5 version: {e}")
 
     deps = [
         # (MODULE_NAME, f(mod) -> mod version)
@@ -98,7 +107,7 @@ def show_versions(as_json=False):
         ("zarr", lambda mod: mod.__version__),
         ("cftime", lambda mod: mod.__version__),
         ("nc_time_axis", lambda mod: mod.__version__),
-        ("PseudonetCDF", lambda mod: mod.__version__),
+        ("PseudoNetCDF", lambda mod: mod.__version__),
         ("rasterio", lambda mod: mod.__version__),
         ("cfgrib", lambda mod: mod.__version__),
         ("iris", lambda mod: mod.__version__),
@@ -108,6 +117,8 @@ def show_versions(as_json=False):
         ("matplotlib", lambda mod: mod.__version__),
         ("cartopy", lambda mod: mod.__version__),
         ("seaborn", lambda mod: mod.__version__),
+        ("numbagg", lambda mod: mod.__version__),
+        ("pint", lambda mod: mod.__version__),
         # xarray setup/test
         ("setuptools", lambda mod: mod.__version__),
         ("pip", lambda mod: mod.__version__),
@@ -118,7 +129,7 @@ def show_versions(as_json=False):
         ("sphinx", lambda mod: mod.__version__),
     ]
 
-    deps_blob = list()
+    deps_blob = []
     for (modname, ver_f) in deps:
         try:
             if modname in sys.modules:
@@ -132,51 +143,18 @@ def show_versions(as_json=False):
                 ver = ver_f(mod)
                 deps_blob.append((modname, ver))
             except Exception:
-                deps_blob.append((modname, 'installed'))
+                deps_blob.append((modname, "installed"))
 
-    if (as_json):
-        try:
-            import json
-        except Exception:
-            import simplejson as json
+    print("\nINSTALLED VERSIONS", file=file)
+    print("------------------", file=file)
 
-        j = dict(system=dict(sys_info), dependencies=dict(deps_blob))
+    for k, stat in sys_info:
+        print(f"{k}: {stat}", file=file)
 
-        if as_json is True:
-            print(j)
-        else:
-            with codecs.open(as_json, "wb", encoding='utf8') as f:
-                json.dump(j, f, indent=2)
-
-    else:
-
-        print("\nINSTALLED VERSIONS")
-        print("------------------")
-
-        for k, stat in sys_info:
-            print("%s: %s" % (k, stat))
-
-        print("")
-        for k, stat in deps_blob:
-            print("%s: %s" % (k, stat))
-
-
-def main():
-    from optparse import OptionParser
-    parser = OptionParser()
-    parser.add_option("-j", "--json", metavar="FILE", nargs=1,
-                      help="Save output as JSON into file, pass in "
-                      "'-' to output to stdout")
-
-    (options, args) = parser.parse_args()
-
-    if options.json == "-":
-        options.json = True
-
-    show_versions(as_json=options.json)
-
-    return 0
+    print("", file=file)
+    for k, stat in deps_blob:
+        print(f"{k}: {stat}", file=file)
 
 
 if __name__ == "__main__":
-    sys.exit(main())
+    show_versions()
